@@ -68,7 +68,7 @@ UL_DEBUG_DEFINE_MASKNAMES(fdisk) = UL_DEBUG_EMPTY_MASKNAMES;
 
 static void fdiskprog_init_debug(void)
 {
-	__UL_INIT_DEBUG(fdisk, FDISKPROG_DEBUG_, 0, FDISK_DEBUG);
+	__UL_INIT_DEBUG_FROM_ENV(fdisk, FDISKPROG_DEBUG_, 0, FDISK_DEBUG);
 }
 
 static void reply_sighandler(int sig __attribute__((unused)))
@@ -271,7 +271,7 @@ static int ask_number(struct fdisk_context *cxt,
 
 	do {
 		int rc = get_user_reply(prompt, buf, bufsz);
-		uint64_t num;
+		uint64_t num = 0;
 
 		if (rc)
 			return rc;
