@@ -67,8 +67,9 @@ struct bfsde {
 	char d_name[BFS_NAMELEN];
 };
 
-static void __attribute__ ((__noreturn__)) usage(FILE * out)
+static void __attribute__((__noreturn__)) usage(void)
 {
+	FILE *out = stdout;
 	fprintf(out,
 		_("Usage: %s [options] device [block-count]\n"),
 		program_invocation_short_name);
@@ -83,12 +84,11 @@ static void __attribute__ ((__noreturn__)) usage(FILE * out)
 		       " -v, --verbose       explain what is being done\n"
 		       " -c                  this option is silently ignored\n"
 		       " -l                  this option is silently ignored\n"
-		       " -V, --version       output version information and exit\n"
-		       "                     -V as version must be only option\n"
-		       " -h, --help          display this help and exit\n\n"));
+		       ));
+	printf(USAGE_HELP_OPTIONS(21));
 
-	fprintf(out, USAGE_MAN_TAIL("mkfs.bfs(8)"));
-	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
+	printf(USAGE_MAN_TAIL("mkfs.bfs(8)"));
+	exit(EXIT_SUCCESS);
 }
 
 static void __attribute__ ((__noreturn__)) print_version(void)
@@ -129,9 +129,10 @@ int main(int argc, char **argv)
 	textdomain(PACKAGE);
 	atexit(close_stdout);
 
-	if (argc < 2)
-		usage(stderr);
-
+	if (argc < 2) {
+		warnx(_("not enough arguments"));
+		errtryhelp(EXIT_FAILURE);
+	}
 	if (argc == 2 && !strcmp(argv[1], "-V"))
 		print_version();
 
@@ -170,32 +171,33 @@ int main(int argc, char **argv)
 		case VERSION_OPTION:
 			print_version();
 		case 'h':
-			usage(stdout);
+			usage();
 		default:
-			usage(stderr);
+			errtryhelp(EXIT_FAILURE);
 		}
 	}
 
-	if (optind == argc)
-		usage(stderr);
+	if (optind == argc) {
+		warnx(_("no device specified"));
+		errtryhelp(EXIT_FAILURE);
+	}
 
 	device = argv[optind++];
 
 	if (stat(device, &statbuf) < 0)
 		err(EXIT_FAILURE, _("stat of %s failed"), device);
 
-	if (!S_ISBLK(statbuf.st_mode))
-		errx(EXIT_FAILURE, _("%s is not a block special device"), device);
-
-	fd = open(device, O_RDWR | O_EXCL);
+	fd = open_blkdev_or_file(&statbuf, device, O_RDWR);
 	if (fd < 0)
 		err(EXIT_FAILURE, _("cannot open %s"), device);
 
 	if (optind == argc - 1)
 		user_specified_total_blocks =
 			strtou64_or_err(argv[optind], _("invalid block-count"));
-	else if (optind != argc)
-		usage(stderr);
+	else if (optind != argc) {
+		warnx(_("bad usage"));
+		errtryhelp(EXIT_FAILURE);
+	}
 
 	if (blkdev_get_sectors(fd, &total_blocks) == -1) {
 		if (!user_specified_total_blocks)
