@@ -199,10 +199,9 @@ static struct monitor_entry *monitor_get_entry(struct libmnt_monitor *mn, int ty
  * Userspace monitor
  */
 
-static int userspace_monitor_close_fd(struct libmnt_monitor *mn,
+static int userspace_monitor_close_fd(struct libmnt_monitor *mn __attribute__((__unused__)),
 				    struct monitor_entry *me)
 {
-	assert(mn);
 	assert(me);
 
 	if (me->fd >= 0)
@@ -313,7 +312,7 @@ static int userspace_event_verify(struct libmnt_monitor *mn,
 
 	DBG(MONITOR, ul_debugobj(mn, "drain and verify userspace monitor inotify"));
 
-	/* the 'fd' is non-blocking */
+	/* the me->fd is non-blocking */
 	do {
 		ssize_t len;
 		char *p;
@@ -326,7 +325,7 @@ static int userspace_event_verify(struct libmnt_monitor *mn,
 		for (p = buf; p < buf + len;
 		     p += sizeof(struct inotify_event) + e->len) {
 
-			int fd;
+			int fd = -1;
 
 			e = (const struct inotify_event *) p;
 			DBG(MONITOR, ul_debugobj(mn, " inotify event 0x%x [%s]\n", e->mask, e->len ? e->name : ""));
@@ -345,7 +344,7 @@ static int userspace_event_verify(struct libmnt_monitor *mn,
 		}
 	} while (1);
 
-	DBG(MONITOR, ul_debugobj(mn, status == 1 ? " success" : " nothing"));
+	DBG(MONITOR, ul_debugobj(mn, "%s", status == 1 ? " success" : " nothing"));
 	return status;
 }
 
@@ -430,10 +429,9 @@ err:
  * Kernel monitor
  */
 
-static int kernel_monitor_close_fd(struct libmnt_monitor *mn,
+static int kernel_monitor_close_fd(struct libmnt_monitor *mn __attribute__((__unused__)),
 				   struct monitor_entry *me)
 {
-	assert(mn);
 	assert(me);
 
 	if (me->fd >= 0)
@@ -863,7 +861,7 @@ err:
 /*
  * create a monitor and add the monitor fd to epoll
  */
-int __test_epoll(struct libmnt_test *ts, int argc, char *argv[], int cleanup)
+static int __test_epoll(struct libmnt_test *ts, int argc, char *argv[], int cleanup)
 {
 	int fd, efd = -1, rc = -1;
 	struct epoll_event ev;
@@ -927,12 +925,12 @@ done:
 /*
  * create a monitor and add the monitor fd to epoll
  */
-int test_epoll(struct libmnt_test *ts, int argc, char *argv[])
+static int test_epoll(struct libmnt_test *ts, int argc, char *argv[])
 {
 	return __test_epoll(ts, argc, argv, 0);
 }
 
-int test_epoll_cleanup(struct libmnt_test *ts, int argc, char *argv[])
+static int test_epoll_cleanup(struct libmnt_test *ts, int argc, char *argv[])
 {
 	return __test_epoll(ts, argc, argv, 1);
 }
@@ -940,7 +938,7 @@ int test_epoll_cleanup(struct libmnt_test *ts, int argc, char *argv[])
 /*
  * create a monitor and wait for a change
  */
-int test_wait(struct libmnt_test *ts, int argc, char *argv[])
+static int test_wait(struct libmnt_test *ts, int argc, char *argv[])
 {
 	const char *filename;
 	struct libmnt_monitor *mn = create_test_monitor(argc, argv);
